@@ -29,6 +29,9 @@
 /* Define as 1 to use hashing */
 #define HASHING 1
 
+/* Define if the preprocessor recognizes __FUNCTION__ */
+#undef HAVE___FUNCTION__
+
 /* Define to the name of the distribution.  */
 #undef PACKAGE
 
