@@ -15,17 +15,35 @@
    version distributed with GoDummy.
 */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #define  _GMP_C_  1
 #include <time.h>
 #include <assert.h>
 #include <sys/types.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
 #include <string.h>
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef __MINGW32__
+#include <windows.h>
+#include <windows32/sockets.h>
+#endif
+
+#ifdef HAVE_VISUAL_C
+#include <winsock.h>
+#include <io.h>
+#endif
+
 /**********************************************************************
  * Constants
  **********************************************************************/
