@@ -2,7 +2,7 @@
  * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
  * http://www.gnu.org/software/gnugo/ for more information.      *
  *                                                               *
- * Copyright 1999 by the Free Software Foundation.               *
+ * Copyright 1999 and 2000 by the Free Software Foundation.      *
  *                                                               *
  * This program is free software; you can redistribute it and/or *
  * modify it under the terms of the GNU General Public License   *
@@ -27,9 +27,10 @@
 #include <stdio.h>
 #include <signal.h>
 
+#include <stdlib.h>
+
 #include "interface.h"
 #include "liberty.h"
-#include "main.h"
 #include "gmp.h"
 #include "sgf.h"
 #include "ttsgf.h"
@@ -212,7 +213,8 @@ void play_gmp()
     */
 
    if (!get_opt_quiet())
-     fprintf(stderr, "Game over - waiting for client to shut us down\n");
+     fprintf(stderr, "Game over---waiting for client to shut us down\n");
+   sgf_close_file();
 
    who_wins(mymove,get_komi(),stderr);
 
