@@ -2,7 +2,7 @@
  * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
  * http://www.gnu.org/software/gnugo/ for more information.      *
  *                                                               *
- * Copyright 1999 by the Free Software Foundation.               *
+ * Copyright 1999 and 2000 by the Free Software Foundation.      *
  *                                                               *
  * This program is free software; you can redistribute it and/or *
  * modify it under the terms of the GNU General Public License   *
@@ -28,18 +28,36 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
+#include <time.h>
+
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 
 #include "interface.h"
 
-#include "main.h"
 #include "sgf.h"
 #include "sgf_properties.h"
 #include "ttsgf_read.h"
 #include "sgfana.h"
 #include "liberty.h"
 
+double gg_gettimeofday(void);
 
+double gg_gettimeofday(void)
+{
+#ifdef HAVE_GETTIMEOFDAY
+  struct timeval tv;
+  gettimeofday(&tv,NULL);
+  return tv.tv_sec + 1.e-6*tv.tv_usec;
+#else
+  return time(NULL);
+#endif
+}
 
 void play_solo(int moves)
 {
@@ -48,7 +66,6 @@ void play_solo(int moves)
   int moval;
   int mymove;
   double t1, t2;
-  struct timeval tv;
   int benchmark = moves;
   
   /* It tends not to be very imaginative in the opening,
@@ -98,8 +115,7 @@ void play_solo(int moves)
 	putc('\n', sgfout);
   */
   
-  gettimeofday(&tv, NULL);
-  t1 = tv.tv_sec + 1.e-6*tv.tv_usec;
+  t1 = gg_gettimeofday();
   
   while (pass < 2 && --moves >= 0 && !time_to_die)
     {
@@ -120,13 +136,16 @@ void play_solo(int moves)
       sgfAnalyzer(moval);
       who=OTHER_COLOR(who);
     }
-  gettimeofday(&tv, NULL);
-  t2 = tv.tv_sec + 1.e-6*tv.tv_usec;
+  t2 = gg_gettimeofday();
   
   /* two passes and it's over */
   
   who_wins(mymove, komi, stdout);
-  printf("%.3f moves/sec\n", (benchmark-moves)/(t2-t1));
+  if (t2 == t1) {
+    printf("%.3f moves played\n", (double) (benchmark-moves));
+  } else {
+    printf("%.3f moves/sec\n", (benchmark-moves)/(t2-t1));
+  }
   
 }
 
@@ -274,6 +293,7 @@ int load_sgf_file(SGFNodeP head, const char *untilstr)
 		    return next;
 		  
 		  put_move(i,j,next);
+		  next = OTHER_COLOR(next);
 		  /* send the move to the file, move_val=0 */
 		  /*  sgf_move_made(i, j, next, 0);        */
 		  /* updateboard(i,j,next);                */
@@ -283,7 +303,7 @@ int load_sgf_file(SGFNodeP head, const char *untilstr)
 	    }
 	}
     }
-  return OTHER_COLOR(next);
+  return next;
 }
 
 /* reads header info from sgf structure and sets the appropriate variables*/
