@@ -2,7 +2,7 @@
  * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
  * http://www.gnu.org/software/gnugo/ for more information.      *
  *                                                               *
- * Copyright 1999 by the Free Software Foundation.               *
+ * Copyright 1999 and 2000 by the Free Software Foundation.      *
  *                                                               *
  * This program is free software; you can redistribute it and/or *
  * modify it under the terms of the GNU General Public License   *
@@ -247,6 +247,8 @@ static int parse_sgf_properties(SGFNodeP node, enum testmode testmode)
       enum quality mq=NF;       /* move quality (eg generated move was on a TR[], CR[], MA[] ) */
 
       int i, j;
+      int move_from_file_x=0;
+      int move_from_file_y=0;
 
       /* get a move from the engine for color 'move' */
       get_test_move(&i, &j, move);
@@ -256,10 +258,69 @@ static int parse_sgf_properties(SGFNodeP node, enum testmode testmode)
       {
 	/* look to see if our move was considered */
 	assert(move_prop);  /* we should not get here otherwise */
-	if (test_move(i,j,move_prop))
-	  mq = FOUND;
-	else if (i >= 0 && i < board_size && j >= 0 && j < board_size && potential_moves[i][j] > 0)
-	  mq = CONSIDERED;
+	move_from_file_x=get_moveX (move_prop);
+	move_from_file_y=get_moveY (move_prop);
+
+	if ((i==move_from_file_x)&&(j==move_from_file_y))
+	  {
+	    mq = FOUND;
+	  }
+	else 
+	  {
+	    if ((i >= 0) && (i < board_size) && (j >= 0) && (j < board_size))
+	      {
+		if (potential_moves[move_from_file_x][move_from_file_y] > 0)
+		  {
+		    mq = CONSIDERED;
+		  }
+		else
+		  {
+		    /* The move at (move_from_file_x,move_from_file_y) was not considered by GNU Go. */
+		    int xx,yy;
+		    printf ("// Start\n");
+		    for (xx=0;xx<board_size;xx++)
+		      {
+			printf ("//  ");
+			for (yy=0;yy<board_size;yy++)
+			  {
+			    if ((xx==move_from_file_x)&&(yy==move_from_file_y))
+			      {
+				if (move==BLACK)
+				  {
+				    printf ("x");
+				  }
+				else
+				  {
+				    printf ("o");
+				  }
+			      }
+			    else
+			      {
+				switch (p[xx][yy])
+				  {
+				  case EMPTY:
+				    printf (".");
+				    break;
+				    
+				  case BLACK:
+				    printf ("X");
+				    break;
+				    
+				  case WHITE:
+				    printf ("O");
+				    break;
+				    
+				  default:
+				    printf ("?");
+				  }
+			      }
+			  }
+			printf ("\n");
+		      }
+		    printf ("// End\n");
+		  }
+	      }
+	  }
       }
       else
       {
@@ -290,6 +351,8 @@ static int parse_sgf_properties(SGFNodeP node, enum testmode testmode)
 	    
 	  case SGFB : 
 	  case SGFW : 
+	    move_from_file_x=get_moveX (sgf_prop);
+	    move_from_file_y=get_moveY (sgf_prop);
 	    /* if the move has already matched an annotation, ignore */
 	    if (mq == NF && test_move(i,j,sgf_prop))
 	      mq=FOUND;  /* genmove matches board move */
@@ -322,14 +385,21 @@ static int parse_sgf_properties(SGFNodeP node, enum testmode testmode)
 	
       /* now report on how well the computer generated the move */
 
-      printf("Computer move: %s (%d): ",
-	     move == WHITE ? "White" : "Black",
-	     get_movenumber());
+      printf("Move %d (%s): ",
+	     get_movenumber()+1,
+	     move == WHITE ? "White" : "Black");
       
-      if(i==get_boardsize() && j==get_boardsize())
-	printf("Pass is ");
+      printf ("GNU Go plays ");
+      if((i==get_boardsize()) && (j==get_boardsize()))
+	printf("PASS. ");
       else
-	mprintf("%m is ",i,j);
+	mprintf("%m. ",i,j);
+
+      printf ("Game move is ");
+      if((move_from_file_x>=get_boardsize()) && (move_from_file_y>=get_boardsize()))
+	printf("PASS. ");
+      else
+	mprintf("%m. ",move_from_file_x,move_from_file_y);
       
       /* if the generated move matches the board move and we have no other
        * information, move quality is node quality except in game mode (?)
@@ -337,7 +407,8 @@ static int parse_sgf_properties(SGFNodeP node, enum testmode testmode)
       
       if (testmode != GAME && mq == FOUND)
 	mq = nq;
-      
+
+      printf ("Result: ");
       switch(mq) {
       case BAD:   printf("BAD");break;
       case OK:    printf("OK");break;
