@@ -2,7 +2,7 @@
  * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
  * http://www.gnu.org/software/gnugo/ for more information.      *
  *                                                               *
- * Copyright 1999 by the Free Software Foundation.               *
+ * Copyright 1999 and 2000 by the Free Software Foundation.      *
  *                                                               *
  * This program is free software; you can redistribute it and/or *
  * modify it under the terms of the GNU General Public License   *
@@ -22,12 +22,13 @@
 
 
 
+/* Helper functions must be declared in patterns.h */
 
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
-#include "../src/liberty.h"
+#include "liberty.h"
 #include "patterns.h"
 
 
@@ -47,6 +48,8 @@
  *
  * The helper must return a weight between 0 and pattern->patwt, plus
  * wind assistance.
+ *
+ * helper functions must also be declared in patterns.h.
  */
 
 
@@ -270,8 +273,9 @@ diag_miai_helper (ARGS)
 
   tval = min(connection_value(ai, aj, bi, bj, ti, tj), COMPUTE_SCORE);
   if (tval > 0
-      && basic_cut_helper(pattern, transformation, ti, tj, color))
+      && basic_cut_helper(pattern, transformation, ti, tj, color)) {
     return tval;
+  }
   else
     return 0;
 }
@@ -328,7 +332,6 @@ diag_miai2_helper (ARGS)
   
   if (!safe_move(ti, tj, acolor))
     return (0);
-
   return tval;
 }
 
@@ -797,7 +800,7 @@ common_geta_helper (ARGS)
   if (worm[ai][aj].attacki != -1)
     return (0);
   if (TRYMOVE(ti, tj, OTHER_COLOR(p[ai][aj]))) {
-    if (savestone2(ai, aj, NULL, NULL)) {
+    if (find_defense(ai, aj, NULL, NULL)) {
       popgo();
       return (0);
     }
@@ -1057,7 +1060,6 @@ try_to_rescue_helper (ARGS)
 {
   int ai, aj;
   int other=OTHER_COLOR(color);
-  int rescue=0;
 
   UNUSED(pattern);
 
@@ -1065,15 +1067,13 @@ try_to_rescue_helper (ARGS)
   if (worm[ai][aj].attacki != -1)
     if (TRYMOVE(ti, tj, other)) {
       if (!attack(ai, aj, NULL, NULL)) {
-	worm[ai][aj].defendi = ti;
-	worm[ai][aj].defendj = tj;
-	rescue=1;
+	popgo();
+	change_defense(ai, aj, ti, tj);
       }
-      popgo();
-      if (rescue)
-	propogate_worm(ai, aj);
+      else
+	popgo();
     }
-  return (0);
+  return COMPUTE_SCORE;
 }
 
 
@@ -1180,7 +1180,7 @@ jump_out_helper (ARGS)
     own_eyespace = (black_eye[ti][tj].color == BLACK_BORDER);
   }
   
-  if (dist!=-1 && dist<=6 && !own_eyespace)
+  if (dist != -1 && dist <= 6 && !own_eyespace)
     return COMPUTE_SCORE;
   else
     return 0;
@@ -1644,7 +1644,6 @@ int
 two_bird_helper (ARGS)
 {
   int ai, aj, bi, bj;
-  int move_attack=0;
 
   OFFSET(0, -1, ai, aj);
   OFFSET(1, -2, bi, bj);
@@ -1653,14 +1652,12 @@ two_bird_helper (ARGS)
       (worm[bi][bj].attacki != -1) && (worm[ai][aj].defendi != -1) &&
       (worm[bi][bj].defendi != -1))
     if (TRYMOVE(ti, tj, color)) {
-      if (!find_defense(bi, bj, NULL, NULL)) {
-	worm[worm[bi][bj].origini][worm[bi][bj].originj].attacki=ti;
-	worm[worm[bi][bj].origini][worm[bi][bj].originj].attackj=tj;
-	move_attack=1;
+      if (!(p[bi][bj] && find_defense(bi, bj, NULL, NULL))) {
+	popgo();
+	change_attack(bi, bj, ti, tj);
       }
-      popgo();
-      if (move_attack)
-	propogate_worm(worm[bi][bj].origini, worm[bi][bj].originj);
+      else
+	popgo();
       return (COMPUTE_SCORE);
     }
   return (0);
@@ -1743,9 +1740,11 @@ break_connection_helper (ARGS)
 
 /*
 
+In this situation the reading code currently doesn't find the move.
+
 ...??              ...??           
 O.*.?              c t.?
-?X...              ?a...
+?X...              ?a.b.
 ?X...              ?a...
 ??O??              ??O??
 
@@ -1754,16 +1753,24 @@ O.*.?              c t.?
 int
 cap3_helper (ARGS)
 {
-  int ai, aj;
-  int tval=0;
+  int ai, aj, bi, bj;
 
   OFFSET(1, -1, ai, aj);
+  OFFSET(1, 1, bi, bj);
+
+  if (worm[ai][aj].attacki != -1)
+    return 0;
+
   if (TRYMOVE(ti, tj, color)) {
-    if (attack(ai, aj, NULL, NULL) && !find_defense(ai, aj, NULL, NULL))
-      tval = COMPUTE_SCORE;
+    if (attack(ai, aj, NULL, NULL) && !find_defense(ai, aj, NULL, NULL)) {
+      popgo();
+      change_attack(ai, aj, ti, tj);
+      change_defense(ai, aj, bi, bj);
+      return  COMPUTE_SCORE;
+    }
     popgo();
   }
-  return tval;
+  return 0;
 }
 
 
@@ -1969,7 +1976,7 @@ no_capture_helper (ARGS)
   OFFSET(1, 1, ai, aj);
   if (worm[ai][aj].liberties==1)
     if (TRYMOVE(ti, tj, color)) {
-      if (!readlad1(ai, aj, NULL, NULL)) 
+      if (!find_defense(ai, aj, NULL, NULL)) 
 	tval=COMPUTE_SCORE;
       popgo();
     }
@@ -2186,7 +2193,7 @@ int
 sente_cut_helper (ARGS)
 {
   int ai, aj, bi, bj, ci, cj;
-  int acolor, ccolor;
+  int ccolor;
   int tval=0;
 
   UNUSED(color);  
@@ -2199,7 +2206,6 @@ sente_cut_helper (ARGS)
     return (0);
 
   ccolor=p[ai][aj];
-  acolor=OTHER_COLOR(ccolor);
   if (TRYMOVE(ti, tj, ccolor)) {
     if ((attack(ai, aj, NULL, NULL) || attack(bi, bj, NULL, NULL)))
       tval=COMPUTE_SCORE;
@@ -2315,7 +2321,7 @@ only_atari_helper (ARGS)
 
   if (((worm[ai][aj].liberties)>1) && (worm[bi][bj].liberties==2)) {
     approxlib(bi, bj, other, 3);
-    if ((libi[0]!=ti) && (libj[0]!=tj)) 
+    if ((libi[0]!=ti) || (libj[0]!=tj)) 
       {
 	ci=libi[0];
 	cj=libj[0];
@@ -2775,7 +2781,6 @@ block_to_kill_helper (ARGS)
 {
   int ai, aj, bi, bj, ci, cj, di, dj;
   int tval=0;
-  int kills=0;
 
   OFFSET(1, 1, ai, aj);
   OFFSET(0, 1, bi, bj);
@@ -2786,14 +2791,12 @@ block_to_kill_helper (ARGS)
     if (!attack(di, dj, NULL, NULL))
       tval=min(connection_value(ai, aj, ci, cj, ti, tj), COMPUTE_SCORE);
     if (attack(ai, aj, NULL, NULL) && !find_defense(ai, aj, NULL, NULL)) {
-      worm[worm[ai][aj].origini][worm[ai][aj].originj].attacki=ti;
-      worm[worm[ai][aj].origini][worm[ai][aj].originj].attackj=tj;
-      kills=1;
+      popgo();
+      change_attack(ai, aj, ti, tj);
     }
-    popgo();
+    else
+      popgo();
   }
-  if (kills)
-    propogate_worm(worm[ai][aj].origini, worm[ai][aj].originj);      
   return tval;
 }
 
@@ -3023,7 +3026,7 @@ pull_back_catch_helper (ARGS)
   if (worm[ai][aj].liberties==2)
     return 0;
   
-  if (dragon[ai][aj].weak==CRITICAL)
+  if (dragon[ai][aj].safety==CRITICAL)
     tval=min(COMPUTE_SCORE, 70);
   if (TRYMOVE(ti, tj, color)) {
     if (TRYMOVE(bi, bj, color)) {
@@ -3056,7 +3059,7 @@ second_line_block_helper (ARGS)
 
   OFFSET(0, 1, ai, aj);
   if ((dragon[ai][aj].status==UNKNOWN) 
-      && (worm[ai][aj].attacki==-1) && (dragon[ai][aj].weak==CRITICAL))
+      && (worm[ai][aj].attacki==-1) && (dragon[ai][aj].safety==CRITICAL))
     return COMPUTE_SCORE;
   return 0;
 }
@@ -3386,6 +3389,7 @@ seki_helper(int ai, int aj)
   return 1;
 }
 
+
 /*
 
 XO     aO
@@ -3408,6 +3412,8 @@ ugly_cutstone_helper (ARGS)
 }
 
 
+
+
 /*
 
 OX              bX
@@ -3417,6 +3423,7 @@ O?              c?
 
 */
 
+
 int
 real_connection_helper (ARGS)
 {
@@ -3437,6 +3444,684 @@ real_connection_helper (ARGS)
 }
 
 
+/*
+
+XO          Xb
+O*          at
+..          ..
+--          --
+
+Here a can be captured, and b is CRITICAL. The helper artificially assigns a 
+genus 1 so that eye_finder will try the connection as a way of saving b. Since
+this is a C pattern, the dragons at a and b become friends.
+
+*/
+
+int
+connect_to_live_helper (ARGS)
+{
+  int ai, aj, bi, bj;
+
+  UNUSED(pattern);
+  UNUSED(color);
+
+  OFFSET(0, -1, ai, aj);
+  OFFSET(-1, 0, bi, bj);
+
+  if ((worm[ai][aj].attacki != -1) 
+      && (dragon[ai][aj].genus == 0) 
+      && (dragon[bi][bj].status == CRITICAL)) {
+    if (dragon[ai][aj].size==1)
+      dragon[ai][aj].genus=1;
+    else {
+      int m, n;
+      for (m=0; m<board_size; m++)
+	for (n=0; n<board_size; n++)
+	  if ((dragon[m][n].origini==dragon[ai][aj].origini)
+	      && (dragon[m][n].originj==dragon[ai][aj].originj))
+	    dragon[m][n].genus=1;
+    }
+  }
+  return 0;
+}
+
+
+/* 
+
+This D type pattern provides a better defense in case the reading
+code recommends something stupid!
+
+*.o?              t.o?
+.O.O              .Oab
+?xOO              ?xOO
+
+*/
+
+int
+stupid_defense_helper (ARGS)
+{
+  int ai, aj, bi, bj;
+
+  UNUSED(pattern);
+
+  OFFSET(1, 2, ai, aj);
+  OFFSET(1, 3, bi, bj);
+
+  if ((worm[bi][bj].defendi == ai)
+      && (worm[bi][bj].defendj == aj)
+      && TRYMOVE(ti, tj, color)) {
+    if (!attack(bi, bj, NULL, NULL)) {
+      popgo();
+      change_defense(bi, bj, ti, tj);
+      return COMPUTE_SCORE;
+    }
+    popgo();
+  }
+  return 0;
+}
+	
+/*
+
+.*O             atd
+OX?             cb?
+
+
+*/
+
+int
+threaten_connect_helper (ARGS)
+{
+  int ai, aj, bi, bj, ci, cj, di, dj;
+  int other=OTHER_COLOR(color);
+
+  OFFSET(0, -1, ai, aj);
+  OFFSET(1, 0, bi, bj);
+  OFFSET(1, -1, ci, cj);
+  OFFSET(0, 1, di, dj);
+
+  if ((worm[bi][bj].attacki != -1) 
+      || (worm[ci][cj].liberties == 1))
+    return 0;
+
+  if ((dragon[ci][cj].origini == dragon[di][dj].origini)
+      && (dragon[ci][cj].originj == dragon[di][dj].originj))
+    return 0;
+
+  if (dragon[ci][cj].safety != ALIVE)
+    return COMPUTE_SCORE;
+
+  if (TRYMOVE(ti, tj, color)) {
+    if (TRYMOVE(ai, aj, other)) {
+      if (attack(ai, aj, NULL, NULL) || attack(bi, bj, NULL, NULL)) {
+	popgo();
+	popgo();
+	return COMPUTE_SCORE;
+      }
+      popgo();
+    }
+    popgo();
+  }
+  return 0;
+}
+
+
+/*
+
+*.O             taO
+OX?             cb?
+
+*/
+
+int
+threaten_connect1_helper (ARGS)
+{
+  int ai, aj, bi, bj, ci, cj;
+  int other=OTHER_COLOR(color);
+
+  OFFSET(0, 1, ai, aj);
+  OFFSET(1, 1, bi, bj);
+  OFFSET(1, 0, ci, cj);
+
+  if (worm[bi][bj].attacki != -1)
+    return 0;
+
+  if ((dragon[ci][cj].safety != ALIVE) && TRYMOVE(ti, tj, color)) {
+    if (TRYMOVE(ai, aj, other)) {
+      if (!attack(ai, aj, NULL, NULL) && !attack(bi, bj, NULL, NULL)) {
+	popgo();
+	popgo();
+	return COMPUTE_SCORE;
+      }
+      popgo();
+    }
+    popgo();
+    return 0;
+  }
+  return 0;
+}
+
+
+/*
+
+This pattern tries to supply eyes which are missed by compute_eyes.
+
+OO.              OOb
+O.*              cat
+XOO              XdO
+
+*/
+
+int
+make_eye_helper (ARGS)
+{
+  int ai, aj, bi, bj, ci, cj, di, dj;
+  int ocolor, xcolor;
+  
+  UNUSED(pattern);
+  UNUSED(color);
+  
+  OFFSET(0, -1, ai, aj);
+  OFFSET(-1, 0, bi, bj);
+  OFFSET(0, -2, ci, cj);
+  OFFSET(1, -1, di, dj);
+
+  ocolor=p[di][dj];
+  xcolor=OTHER_COLOR(ocolor);
+
+  if (((ocolor==WHITE) && (white_eye[ai][aj].maxeye != 0))
+      || ((ocolor==BLACK) && (black_eye[ai][aj].maxeye != 0)))
+    return 0;
+
+  if (TRYMOVE(ti, tj, ocolor)) {
+    if (!safe_move(bi, bj, xcolor)) {
+      popgo();
+      retrofit_half_eye(ti, tj, ci, cj);
+      return COMPUTE_SCORE;
+    }
+    popgo();
+  }
+  return 0;
+}
+
+      
+/*
+
+This pattern tries to supply eyes which are missed by compute_eyes.
+Here the topological eye criterion can erroneously classify the
+eye at d as FALSE. If it is actually a half eye, we must make
+adjustments.
+
+.OX                 bOX
+*.O                 tde
+.Oo                 cOa
+
+*/
+
+int
+make_eye1_helper (ARGS)
+{
+  int ai, aj, bi, bj, ci, cj, di, dj, ei, ej;
+  int ocolor, xcolor;
+
+  UNUSED(pattern);
+  UNUSED(color);
+
+  OFFSET(1, 2, ai, aj);
+  OFFSET(-1, 0, bi, bj);
+  OFFSET(1, 0, ci, cj);
+  OFFSET(0, 1, di, dj);
+  OFFSET(0, 2, ei, ej);
+
+  ocolor=p[ei][ej];
+  xcolor=OTHER_COLOR(ocolor);
+
+  if ((ocolor==WHITE) && white_eye[di][dj].maxeye>0)
+    return 0;
+  if ((ocolor==BLACK) && black_eye[di][dj].maxeye>0)
+    return 0;
+
+  if ((p[ai][aj]==ocolor) || !safe_move(ai, aj, xcolor))
+    if (TRYMOVE(ti, tj, ocolor)) {
+      if (!safe_move(bi, bj, xcolor) && !safe_move(ci, cj, xcolor)) {
+	popgo();
+	retrofit_half_eye(ti, tj, ei, ej);
+	return COMPUTE_SCORE;
+      }
+      else popgo();
+    }
+  return 0;
+}
+
+
+/*
+
+This should be classified as
+
+ .
+.X!
+
+but sometimes it is not recognised as O eyespace at all. The
+helper tries to correct this.
+
+oOo?                dbe?
+O.OX                O.OX
+.X*.                act.
+----                ----
+
+*/
+
+int
+make_eye2_helper (ARGS)
+{
+  int ai, aj, bi, bj, ci, cj, di, dj, ei, ej;
+  int ocolor, xcolor;
+
+  UNUSED(pattern);
+  UNUSED(color);
+
+  OFFSET(0, -2, ai, aj);
+  OFFSET(-2, -1, bi, bj);
+  OFFSET(0, -1, ci, cj);
+  OFFSET(-2, -2, di, dj);
+  OFFSET(-2, 0, ei, ej);
+
+  ocolor=p[bi][bj];
+  xcolor=OTHER_COLOR(ocolor);
+
+  if ((p[di][dj] == EMPTY) && safe_move(di, dj, xcolor))
+    return 0;
+
+  if ((p[ei][ej] == EMPTY) && safe_move(ei, ej, xcolor))
+    return 0;
+
+  if (((ocolor == WHITE) && (white_eye[ci][cj].origini == -1))
+      || ((ocolor == BLACK) && (black_eye[ci][cj].origini == -1)))
+    if (!safe_move(ai, aj, xcolor))
+      retrofit_half_eye(ti, tj, bi, bj);
+  return 0;
+}
+
+
+/*
+
+oO*ooo?               oatooo?
+O...ooo               bc..ooo
+-------               -------
+
+*/
+
+int
+make_eye3_helper (ARGS)
+{
+  int ai, aj, bi, bj, ci, cj;
+  int ocolor;
+
+  UNUSED(pattern);
+  UNUSED(color);
+
+  OFFSET(0, -1, ai, aj);
+  OFFSET(1, -2, bi, bj);
+  OFFSET(1, -1, ci, cj);
+
+  ocolor=p[ai][aj];
+  if (((ocolor == WHITE) 
+       && (white_eye[ci][cj].esize == 3)
+       && (white_eye[ci][cj].marginal_neighbors == 2))
+      ||((ocolor == BLACK) 
+	 && (black_eye[ci][cj].esize == 3)
+	 && (black_eye[ci][cj].marginal_neighbors == 2))) {
+    retrofit_half_eye(ti, tj, ai, aj);
+    return COMPUTE_SCORE;
+  }
+  return 0;
+}
+
+
+
+/*
+
+If it is possible to hane in this shape, eye_finder frequently
+underestimates the eye potential. This pattern adds a half eye
+to try to live.
+
+
+xXO.             xXab
+.*..             .t..
+....             ....
+----             ----
+
+*/
+
+int
+expand_eyespace_helper (ARGS)
+{
+  int ai, aj, bi, bj;
+
+  UNUSED(pattern);
+  UNUSED(color);
+
+  OFFSET(-1, 1, ai, aj);
+  OFFSET(-1, 2, bi, bj);
+  
+  if ((dragon[ai][aj].genus == 1) 
+      && (dragon[ai][aj].heyes == 0)
+      && (((p[ai][aj]==WHITE) 
+	   && (white_eye[bi][bj].dragoni == dragon[ai][aj].origini)
+	   && (white_eye[bi][bj].dragoni == dragon[ai][aj].origini)
+	   && (white_eye[bi][bj].marginal == 0))
+	  ||((p[ai][aj]==BLACK) 
+	     && (black_eye[bi][bj].dragoni == dragon[ai][aj].origini)
+	     && (black_eye[bi][bj].dragoni == dragon[ai][aj].origini)
+	     && (black_eye[bi][bj].marginal == 0))))
+    retrofit_half_eye(ti, tj, ai, aj);
+  return 0;
+}
+
+
+/*
+
+X???            X???
+.OO?            .bO?
+...X            a..X
+.*.?            .t.?
+
+The cap is overlooked by the reading code.
+
+*/
+
+int
+cap_kills_helper (ARGS)
+{
+  int ai, aj, bi, bj;
+  int other=OTHER_COLOR(color);
+
+  OFFSET(-2, 0, bi, bj);
+  OFFSET(-1, -1, ai, aj);
+
+  if ((worm[bi][bj].liberties>3) || (worm[bi][bj].attack_code==1))
+    return 0;
+
+  if (TRYMOVE(ai, aj, other)) {
+    if (attack(bi, bj, NULL, NULL) && !find_defense(bi, bj, NULL, NULL)) {
+      popgo();
+      return COMPUTE_SCORE;
+    }
+    popgo();
+  }
+  return 0;
+}
+
+
+/*
+
+?O.             ?Ob
+X*O             Xta
+?O.             ?Oc
+
+*/
+
+
+int
+chimera_helper (ARGS)
+{
+  int ai, aj, bi, bj, ci, cj;
+  int m, n;
+  int ocolor;
+  int status;
+
+  UNUSED(pattern);
+  UNUSED(color);
+
+  OFFSET(0, 1, ai, aj);
+  OFFSET(-1, 1, bi, bj);
+  OFFSET(1, 1, ci, cj);
+
+  ocolor=p[ai][aj];
+
+  if ((dragon[ai][aj].heyes == 2)
+      && (dragon[ai][aj].genus == 0)
+      && (half_eye[bi][bj].type == HALF_EYE)
+      && (half_eye[bi][bj].ki == ti)
+      && (half_eye[bi][bj].kj == tj)
+      && (half_eye[ci][cj].type == HALF_EYE)
+      && (half_eye[ci][cj].ki == ti)
+      && (half_eye[ci][cj].kj == tj)) {
+    
+    for (m=0; m<board_size; m++)
+      for (n=0; n<board_size; n++)
+	if ((dragon[m][n].origini == dragon[ai][aj].origini)
+	    && (dragon[m][n].originj == dragon[ai][aj].originj)) {
+	  dragon[m][n].heyes = 1;
+	  dragon[m][n].genus = 1;
+	  dragon[m][n].heyei = ti;
+	  dragon[m][n].heyej = tj;
+	}
+    status=dragon_status(ai, aj);
+    for (m=0; m<board_size; m++)
+      for (n=0; n<board_size; n++)
+	if ((dragon[m][n].origini == dragon[ai][aj].origini)
+	    && (dragon[m][n].originj == dragon[ai][aj].originj))
+	  dragon[m][n].status = status;
+
+  }
+  return 0;
+}
+
+
+/*
+
+--+
+*.|
+O.|
+?O|
+
+*/
+
+int
+semimarginal_helper (ARGS)
+{
+  int ai, aj;
+  int ocolor;
+
+  UNUSED(pattern);
+  UNUSED(color);
+
+  OFFSET(1, 0, ai, aj);
+  ocolor = p[ai][aj];
+
+  if ((((ocolor==WHITE)
+	&& (white_eye[ti][tj].esize==3)
+	&& (white_eye[ti][tj].msize==1)
+	&& (white_eye[ti][tj].marginal))
+       ||
+       ((ocolor==BLACK)
+	&& (black_eye[ti][tj].esize==3)
+	&& (black_eye[ti][tj].msize==1)
+	&& (black_eye[ti][tj].marginal)))
+      && (dragon[ai][aj].genus > 0)) {
+    retrofit_half_eye(ti, tj, ai, aj);
+    retrofit_genus(ai, aj, -1);
+  }
+  return 0;
+}
+
+
+/*
+
+.O.O               cb.O
+O.X*               O.at
+----               ----
+
+*/
+
+int
+eye_on_edge_helper (ARGS)
+{
+  int ai, aj, bi, bj, ci, cj;
+  int ocolor;
+
+  UNUSED(pattern);
+  UNUSED(color);
+
+  OFFSET(0, -1, ai, aj);
+  OFFSET(-1, -2, bi, bj);
+  OFFSET(-1, -3, ci, cj);
+  ocolor=p[bi][bj];
+
+  if ((worm[ai][aj].attacki == -1) 
+      || (worm[ai][aj].defendi == -1))
+    return 0;
+
+  if ((((ocolor==WHITE) && (white_eye[ai][aj].origini == -1))
+       || ((ocolor==BLACK) && (black_eye[ai][aj].origini == -1)))
+      && (does_attack(ti, tj, ai, aj))) {
+    retrofit_half_eye(ti, tj, bi, bj);
+    if (((ocolor==WHITE) && (white_eye[ci][cj].marginal))
+	|| ((ocolor==BLACK) && (black_eye[ci][cj].marginal)))
+      retrofit_genus(bi, bj, 1);
+  }
+  return 0;
+}
+
+
+/*
+
+?..?          ?a.?
+OX*.          dXt.
+XOXO          cebO
+
+*/
+
+int
+make_trouble_helper (ARGS)
+{
+  int ai, aj, bi, bj, ci, cj, di, dj, ei, ej;
+  int ui, uj;
+  int other=OTHER_COLOR(color);
+  int can_attack_b=0;
+  int can_defend_e=0;
+  int can_attack_c=0;
+
+  UNUSED(pattern);
+
+  OFFSET(-1, -1, ai, aj);
+  OFFSET(1, 0, bi, bj);
+  OFFSET(1, -2, ci, cj);
+  OFFSET(0,-2, di, dj);
+  OFFSET(1, -1, ei, ej);
+
+  if ((worm[di][dj].liberties < 2) || (worm[ei][ej].liberties <2))
+    return 0;
+
+  if (worm[bi][bj].liberties==1)
+    return 0;
+
+  if (TRYMOVE(ti, tj, color)) {
+    if (TRYMOVE(ai, aj, other)) {
+      if (attack(bi, bj, &ui, &uj)) {
+	can_attack_b=1;
+	if (worm[ci][cj].attacki != -1) 
+	  if (TRYMOVE(ui, uj, color)) {
+	    if ((p[ci][cj] == EMPTY)
+		|| !find_defense(ci, cj, NULL, NULL)) {
+	      can_attack_c=1;
+	      if ((p[ei][ej] != EMPTY) && 
+		  worm[ei][ej].attacki != -1)
+		can_defend_e=1;
+	    }
+	    popgo();
+	  }
+      }
+      popgo();
+    }
+    popgo();
+  }
+
+  /* The functions change_attack and change_defense must not be called
+   * with stackp>0.
+   */
+
+  if (can_attack_b) {
+    change_attack(bi, bj, ti, tj);
+    if (singleton(ci, cj))
+      worm[ci][cj].value=76;
+  }
+  if (can_attack_c)
+    change_attack(ci, cj, ti, tj);
+  if (can_defend_e)
+    change_defense(ei, ej, ti, tj);
+
+  return 0;
+}
+
+
+/* 
+
+A typical situation:
+
+XXXX
+XOOOXXXX
+Xt.OOOOX
+..u...OX
+--------
+
+Here the eye pattern !.@... is found but gives an incorrect move
+at u in this example. The right move is at t.
+
+?O?                 cad
+*.O                 tbO
+...                 .e.
+---                 ---
+
+So if a half eye is found at e, yet t is a safe move for x,
+the half eye point is moved to t.
+
+*/
+
+
+
+int
+move_half_eye_helper (ARGS)
+{
+  int ai, aj, bi, bj, ci, cj, di, dj, ei, ej;
+  int m, n;
+  int ocolor, xcolor;
+  int move_eye=0;
+
+  UNUSED(pattern);
+  UNUSED(color);
+
+  OFFSET(-1, 1, ai, aj);
+  OFFSET(0, 1, bi, bj);
+  OFFSET(-1, 0, ci, cj);
+  OFFSET(-1, 2, di, dj);
+  OFFSET(1, 1, ei, ej);
+
+  ocolor=p[ai][aj];
+  xcolor=OTHER_COLOR(ocolor);
+  
+  if (((p[ci][cj]==ocolor) || !safe_move(ci, cj, xcolor))
+      && ((p[di][dj]==ocolor) || !safe_move(di, dj, xcolor))
+      && (dragon[ai][aj].heyei==ei)
+      && (dragon[ai][aj].heyej==ej)
+      && (safe_move(ti, tj, xcolor)))
+    if (TRYMOVE(ti, tj, ocolor)) {
+      if (!safe_move(ei, ej, xcolor))
+	move_eye=1;
+      popgo();
+    }
+  if (move_eye)
+    for (m=0; m<board_size; m++)
+      for (n=0; n<board_size; n++)
+	if ((dragon[m][n].origini==dragon[ai][aj].origini)
+	    && (dragon[m][n].originj==dragon[ai][aj].originj)) {
+	  dragon[m][n].heyei=ti;
+	  dragon[m][n].heyej=tj;
+	}
+  return 0;
+}
+
+
 
 /*
  * LOCAL Variables:
@@ -3444,3 +4129,4 @@ real_connection_helper (ARGS)
  * c-basic-offset: 2
  * End:
  */
+
