@@ -2,7 +2,7 @@
  * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
  * http://www.gnu.org/software/gnugo/ for more information.      *
  *                                                               *
- * Copyright 1999 by the Free Software Foundation.               *
+ * Copyright 1999 and 2000 by the Free Software Foundation.      *
  *                                                               *
  * This program is free software; you can redistribute it and/or *
  * modify it under the terms of the GNU General Public License   *
@@ -26,10 +26,10 @@
 
 
 /* Compile the patterns, halfeyes, or connections database. This
- * produces patterns.c, hey.c or conn.c
+ * produces patterns.c or conn.c
  */
 
-/* See also patterns.h, patterns.db, hey.db and conn.db. */
+/* See also patterns.h, patterns.db and conn.db. */
 
 
 /* Differences when compiling half-eye patterns (-h) :
@@ -78,9 +78,8 @@ Usage : mkpat [-cvh] <prefix>\n\
 #include <ctype.h>
 #include <assert.h>
 
-#include <unistd.h>
-
 #include "patterns.h"
+#include "getopt.h"
 
 
 #define DEBUG(x)  /* printf x */
@@ -154,7 +153,7 @@ static struct autohelper_func autohelper_functions[] = {
   {"does_attack",     2, "does_attack(%ci,%cj,%ci,%cj)"},
   {"attack",          1, "(worm[%ci][%cj].attacki != -1)"},
   {"defend",          1, "(worm[%ci][%cj].defendi != -1)"},
-  {"weak",            1, "(dragon[%ci][%cj].weak==CRITICAL)"},
+  {"weak",            1, "(dragon[%ci][%cj].safety==CRITICAL)"},
   {"safe_xmove",      1, "safe_move(%ci,%cj,OTHER_COLOR(color))"},
   {"safe_omove",      1, "safe_move(%ci,%cj,color)"},
   {"xmoyo",           1, "(moyo_color(%ci,%cj) == OTHER_COLOR(color))"},
@@ -164,6 +163,7 @@ static struct autohelper_func autohelper_functions[] = {
   {"xterri",          1, "(terri_color(%ci,%cj) == OTHER_COLOR(color))"},
   {"oterri",          1, "(terri_color(%ci,%cj) == color)"},
   {"genus",           1, "dragon[%ci][%cj].genus"},
+  {"cutstone",        1, "worm[%ci][%cj].cutstone"},
   {"xlib",            1, "countlib(%ci,%cj,OTHER_COLOR(color))"},
   {"olib",            1, "countlib(%ci,%cj,color)"},
   {"xcut",            1, "cut_possible(%ci,%cj,OTHER_COLOR(color))"},
@@ -201,15 +201,6 @@ edit it directly. Instead, edit the connection database. */\n\
 #include \"patterns.h\"\n\
 "
 
-#define HEY_PREAMBLE "\
-/* This file is automatially generated by mkpat. Do not\n\
-edit it directly. Instead, edit the eye database. */\n\
-\n\
-#include <stdio.h> /* for NULL */\n\
-#include \"liberty.h\"\n\
-#include \"patterns.h\"\n\
-"
-
 #define PAT_PREAMBLE "\
 /* This file is automatially generated by mkpat. Do not\n\
 edit it directly. Instead, edit the pattern database. */\n\
@@ -616,8 +607,6 @@ finish_pattern(char *line)
   }
 
   /* translate posn of * to relative co-ords
-   * But for hey database, 0,0 means no move, ie half-eye,
-   * so leave it alone
    */
 
   if (num_stars == 1) {
@@ -714,6 +703,7 @@ finish_pattern(char *line)
       if (strchr(class,'n')) pattern[patno].class |= CLASS_n;
       if (strchr(class,'B')) pattern[patno].class |= CLASS_B;
       if (strchr(class,'A')) pattern[patno].class |= CLASS_A;
+      if (strchr(class,'L')) pattern[patno].class |= CLASS_L;
     }
 
     switch (pattern_type) {
@@ -950,7 +940,7 @@ finish_constraint(void)
 	break;
 
       case 2: /* Time for a label. */
-	if (!strchr(VALID_CONSTRAINT_LABELS, *p)) {
+	if ((*p != '*') && !strchr(VALID_CONSTRAINT_LABELS, *p)) {
 	  if (strchr("XxOo", *p))
 	    fprintf(stderr,
 		    "Error, '%c' is not allowed as a constraint label (pattern %s).\n",
@@ -962,12 +952,16 @@ finish_constraint(void)
 	  return;
 	}
 	else {
-	  label = (int) *p;
-	  if (label_coords[label][0] == -1) {
-	    fprintf(stderr,
-		    "Error, The constraint uses a label (%c) that wasn't specified in the diagram (pattern %s).\n", 
-		    label, pattern_names[patno]);
-	    return;
+	  if (*p == '*')
+	    label = (int) 't';
+	  else {
+	    label = (int) *p;
+	    if (label_coords[label][0] == -1) {
+	      fprintf(stderr,
+		      "Error, The constraint uses a label (%c) that wasn't specified in the diagram (pattern %s).\n", 
+		      label, pattern_names[patno]);
+	      return;
+	    }
 	  }
 	  labels[number_of_params] = label;
 	  number_of_params++;
@@ -1243,9 +1237,6 @@ main(int argc, char *argv[])
   case PATTERNS:
     printf(PAT_PREAMBLE);
     break;
-  case HALFEYES:
-    printf(HEY_PREAMBLE);
-    break;
   case CONNECTIONS:
     printf(CONN_PREAMBLE);
     break;
