@@ -2,7 +2,7 @@
  * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
  * http://www.gnu.org/software/gnugo/ for more information.      *
  *                                                               *
- * Copyright 1999 by the Free Software Foundation.               *
+ * Copyright 1999 and 2000 by the Free Software Foundation.      *
  *                                                               *
  * This program is free software; you can redistribute it and/or *
  * modify it under the terms of the GNU General Public License   *
@@ -116,6 +116,7 @@ typedef int (*pattern_helper_fn_ptr)(struct pattern *, int rotation, int ti, int
 #define CLASS_n 0x80  /* X could also make this move if we do not */
 #define CLASS_B 0x100 /* move breaks connection between enemy worms */
 #define CLASS_A 0x200 /* change attack point of a worm */
+#define CLASS_L 0x400 /* run pattern regardless of weight */
 
 /* methods of assistance */
 
@@ -265,14 +266,29 @@ DECLARE(atari_ladder_helper);
 DECLARE(safe_cut_helper);
 DECLARE(waist_helper);
 DECLARE(ugly_cutstone_helper);
+DECLARE(cutstone2_helper);
 DECLARE(real_connection_helper);
+DECLARE(connect_to_live_helper);
+DECLARE(stupid_defense_helper);
+DECLARE(threaten_connect_helper);
+DECLARE(threaten_connect1_helper);
+DECLARE(make_eye_helper);
+DECLARE(make_eye1_helper);
+DECLARE(make_eye2_helper);
+DECLARE(make_eye3_helper);
+DECLARE(expand_eyespace_helper);
+DECLARE(cap_kills_helper);
+DECLARE(chimera_helper);
+DECLARE(semimarginal_helper);
+DECLARE(eye_on_edge_helper);
+DECLARE(make_trouble_helper);
+DECLARE(move_half_eye_helper);
 
 /* autohelper fns */
 int seki_helper(int ai, int aj);
 
 /* pattern arrays themselves */
 extern struct pattern pat[];
-extern struct pattern hey[];
 extern struct pattern conn[];
 
 /* not sure if this is the best way of doing this, but... */
